﻿var verify = require("../util/verify");
var log = require("../util/log")(module);
var file = require('../util/file');
var sourcetree = require('./helpers/sourcetree');
var ProductionContext = require('./context');

var ProductionService = function (options) {
    this.options = options;
    this.schemaModules = [];
    this.artifactModules = [];
    this.productionContext = new ProductionContext();
    log.setLevel(options.verbose || 0);
};

// register a schema module
ProductionService.prototype.registerSchemaModule = function (schemaModule) {
    this.schemaModules.push(schemaModule);
}

// get a schema loader
ProductionService.prototype.getSchemaLoader = function (schemaName) {
    // console.log('ProductionService.getSchemaLoader.schemaName: ' + schemaName);
    var result = null;
    this.schemaModules.forEach(function (item) {
        if (result == null) {
            if (verify.isFunction(item[schemaName])) result = item[schemaName];
        }
    });
    if (!result) {
        log.error('getSchemaLoader. Cannot find schema loader: ' + schemaName);
        var availables = [];
        this.schemaModules.forEach(function (item) {
            for (key in item) {
                if (verify.isFunction(item[key])) availables.push(key);
            }
        });
        log.error('availables: ' + availables.join(', '));
    }
    return result;
}

// register an artifact module
ProductionService.prototype.registerArtifactModule = function (artifactModule) {
    this.artifactModules.push(artifactModule);
}

// get an artifact generator
ProductionService.prototype.getArtifactGenerator = function (artifactName) {
    // console.log('ProductionService.getArtifactGenerator.artifactName: ' + artifactName);
    var result = null;
    this.artifactModules.forEach(function (item) {
        if (result == null) {
            if (result == null) {
                if (verify.isFunction(item[artifactName])) result = item[artifactName];
            }
        }
    });
    if (!result) {
        log.error('getArtifactGenerator. Cannot find artifact generator: ' + artifactName);
    }
    return result;
}

ProductionService.prototype.loadWmtModel = function (schemaName, ittfSource, context, callback) {

    var schemaNameDef = file.exists(ittfSource) ? schemaName : schemaName + 'FromString';

    var schemaLoader = this.getSchemaLoader(schemaNameDef);
    if (schemaLoader == null) {
        callback({ message: 'ProductionService.loadWmtModel error. Cannot find schema loader for schema ' + schemaName });
        return;
    }
    schemaLoader(ittfSource, context, function (err, result) {
        callback(err, result);
    });

}

ProductionService.prototype.loadWmtSourceTree = function (schemaName, ittfSource, context, callback) {

    var schemaNameDef = file.exists(ittfSource) ? schemaName : schemaName + 'FromString';

    var schemaLoader = this.getSchemaLoader(schemaNameDef);
    if (schemaLoader == null) {
        callback({ message: 'ProductionService.loadWmtModel error. Cannot find schema loader for schema ' + schemaName });
        return;
    }

    var self = this;
    schemaLoader(ittfSource, context, function (err, result) {
        callback(err, sourcetree(result, file.exists(ittfSource) ? ittfSource : null, context.wzrepoRootPath));
    });

}


ProductionService.prototype.generateArtifact = function (artifactName, ittfSource, context, callback) {
    if (verify.isString(artifactName) === false) {
        callback({ message: 'ProductionService.generateArtifact error. Invalid artifact name ' + artifactName });
        return;
    }
    var ss = artifactName.split('/');
    if (ss.length == 2) {
        artifactName = ss[0] + ss[1].substr(0, 1).toUpperCase() + ss[1].substr(1);
    }
    var artifactNameDef = file.exists(ittfSource) ? artifactName : artifactName + 'FromString';
    var artifactGenerator = this.getArtifactGenerator(artifactNameDef);
    if (artifactGenerator == null) {
        callback({ message: 'ProductionService.generateArtifact error. Cannot find artifact generator for artifact ' + artifactName });
        return;
    }
    artifactGenerator(ittfSource, context, function (err, result) {
        callback(err, result);
    });
}

module.exports = ProductionService;
